import time

from .logger import logger


def addlogger(name: str):
    logger.add(
        sink=f"../logs/exploits/{name}.log",
        format="<d>{time:HH:mm:ss.SS}</d> | <level>{level:^8}</level> | {message}",
        enqueue=True,
        filter=lambda record: record["extra"]["file"] == f"{name}.log",
        level="DEBUG",
        colorize=True,
    )


def launchAttack(exploit, target_ip: str, gametickQueue):
    log = logger.bind(file=f"exploits.log")
    gametickQueue.get()
    level = "INFO"
    start = time.time()
    flags_iterable = exploit(target_ip)
    if isinstance(flags_iterable, str):
        flags_iterable: list[str] = [flags_iterable]
    for flag in flags_iterable:
        save_flags(flag, exploit.__module__)  # type: ignore

    elapsed = time.time() - start
    gametickQueue.task_done()

    log.log(level, f"{exploit.__module__} took {elapsed:.2f} seconds")
    return



import importlib
from multiprocessing import JoinableQueue
import os
import sys
from threading import Thread
import click
from pathlib import Path

from .exploits import addlogger, launchAttack

with open(Path(Path(__file__).parent, "template.py")) as fs:
    temp = fs.read()


@click.group()
def app():
    """Simple program to aid in the development and testing of exploits."""
    # click.echo(f"Hello!")


@app.command()
@click.argument("filename", required=False, type=click.Path())
def template(filename):
    """Generate a template for your exploit that is compatible with M1lkman"""
    confirmed = True

    if not filename:
        click.echo("No filename specified")
        filename = "exploit_template.py"

    if Path(filename).exists():
        confirmed = click.confirm(
            f"File {filename} already exists. Do you want to overwrite it?",
            default=False,
            show_default=True,
        )

    if confirmed:
        click.echo(f"Generating file {filename} from template")
        with open(filename, "w") as fs:
            fs.write(temp)


@app.command()
@click.option(
    "--target",
    "-t",
    type=click.STRING,
    default="0.0.0.0",
    show_default=True,
    prompt="Target IP",
)
@click.argument("filename", required=True, type=click.Path(exists=True))
def test(target, filename):
    """Test your exploit against a target ip"""
    if os.system("ping -c 1 " + target + " > /dev/null") != 0:
        click.echo(f"Host {target} is unreachable")
        return
    click.echo(f"Testing {filename} on {target}")
    filename = Path(filename)
    name = filename.stem

    try:
        spec = importlib.util.spec_from_file_location(name, filename.resolve())
        module = importlib.util.module_from_spec(spec)  # type: ignore
        sys.modules[name] = module
        spec.loader.exec_module(module)  # type: ignore
        exploitfun = getattr(sys.modules[name], "main")
    except Exception as e:
        raise e(f"Error while importing {name}")

    gametickQueue = JoinableQueue()
    addlogger(name)
    t = Thread(
        target=launchAttack,
        args=(
            exploitfun,
            target,
            gametickQueue,
        ),
        name=f"{exploitfun.__module__}_{target}",
    )
    gametickQueue.put("")
    t.start()
    gametickQueue.join()
    gametickQueue.close()
    t.join()



import sys

from loguru import logger

logger.remove()
_logs_dir = "../logs"
_conf = {
    "handlers": [
        {
            "sink": sys.stdout,
            "enqueue": True,
            "format": "<d>{time:HH:mm:ss.SS}</d> <c>{name}</c> <level>[{level:^8}]</level> : {message}",
            "level": "DEBUG",
            "colorize": True,
        },
        {
            "sink": f"{_logs_dir}/db.log",
            "format": "<d>{time:HH:mm:ss.SS}</d> | <level>{level:<8}</level>\t{message}",
            "enqueue": True,
            "filter": lambda record: record["extra"]["file"] == "db.log",
            "level": "DEBUG",
            "colorize": True,
        },
        {
            "sink": f"{_logs_dir}/exploits.log",
            "format": "<d>{time:HH:mm:ss.SS}</d> | <level>{level:<8}</level>\t{message}",
            "enqueue": True,
            "filter": lambda record: record["extra"]["file"] == "exploits.log",
            "level": "DEBUG",
            "colorize": True,
        },
        {
            "sink": f"{_logs_dir}/observer.log",
            "format": "<d>{time:HH:mm:ss.SS}</d> | <level>{level:<8}</level>\t{message}",
            "enqueue": True,
            "filter": lambda record: record["extra"]["file"] == "observer.log",
            "level": "DEBUG",
            "colorize": True,
        },
        {
            "sink": f"{_logs_dir}/gameloop.log",
            "format": "<d>{time:HH:mm:ss.SS}</d> | <level>{level:<8}</level>\t{message}",
            "enqueue": True,
            "filter": lambda record: record["extra"]["file"] == "gameloop.log",
            "level": "DEBUG",
            "colorize": True,
        },
        {
            "sink": f"{_logs_dir}/flagsubmitter.log",
            "format": "<d>{time:HH:mm:ss.SS}</d> | <level>{level:<8}</level>\t{message}",
            "enqueue": True,
            "filter": lambda record: record["extra"]["file"] == "flagsubmitter.log",
            "level": "DEBUG",
            "colorize": True,
        },
    ],
}

logger.configure(**_conf)
logger = logger.bind(file="")
